(defpackage :sectool
  (:use :common-lisp :system :i18n :ext)
  (:export #:debugmsg #:report #:test_exit #:envvar #:initenv #:initconst
	   :+E_OK+ :+E_FAIL+ :+E_FATAL+)
  (:documentation "secTool support functions for Common Lisp"))
(in-package :sectool)

(defconstant +package-name+ :sectool)
(defconstant +E_OK+    0)
(defconstant +E_FAIL+  1)
(defconstant +E_FATAL+ 2)
(defconstant +MYDOMAIN+ "sectool")

(defconstant +report-formats+
  '(:WARNING "W~2,'0D : "
    :ERROR   "E~2,'0D : "
    :HINT    "H~2,'0D : "
    :INFO    "I : "))

(defmacro _ (msgid)
  `(gettext ,msgid +MYDOMAIN+))

(defmacro debugmsg (&body body)
  `(format *error-output* ,@body))

(defmacro report (type arg1 &optional arg2 &rest args)
  (let ((fmt (or (getf +report-formats+ type) (error (_ "Bad parameter to report: ~A") type))))
    `(format *standard-output*
	     ,@(if (eql type :INFO)
		   `(,(string-concat fmt (_ arg1)) ,arg2 ,@args) 
		   `(,(string-concat fmt (_ arg2)) ,arg1 ,@args)))))

(defmacro test_exit (ret &optional message)
  (let ((exit-code (if (symbolp ret) (symbol-value ret) ret)))
  `(progn 
     ,(if (= exit-code 0)
	  `(format *standard-output* (_ "Test: ~A, Succeeded~%") COMMON-LISP-USER::+NAME+)
	  `(format *error-output* (_ "Test: ~A, Error: ~D, - ~A~%") COMMON-LISP-USER::+NAME+ ,exit-code ,message))
     (exit ,exit-code))))

(defun envvar (name &optional default)
  (or (ext:getenv name) default))

(defmacro create-symbol (&rest strings)
  `(read-from-string (string-concat ,@strings)))

(defmacro initconst (&rest symbols)
  "Imports constants from package"
  (let ((clist nil))
    (dolist (sym symbols)
      (push (create-symbol (symbol-name +package-name+) ":" (symbol-name sym)) clist))
    `(shadowing-import ',clist)))

(defmacro initenv (&rest vars)
  "Initialization of enviroment variables"
  (let ((clist nil))
    (dolist (varname vars)
      (push `(defconstant ,(create-symbol "+" varname "+") ,(envvar varname "")) clist))
    `(progn ,@clist)))
