#!/usr/bin/env sh
if (( ${DEBUG} == 1 )); then 
. "${TESTS_DIRS}/bash_defs_debug.sh" 
else
. "${TESTS_DIRS}/bash_defs.sh"
fi
# -------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.
# Written by Dan Kopecek <dkopecek@redhat.com>
# Thu Jan 31 14:53:38 CET 2008
# selftest.sh
# TODO: fix stupid regexps
# -------------------------------- #

# --- Constants ------------------ #
const E_NOFILE=2
const E_NORECORD=503
const E_NOLIST=504
const E_DIFF=505
const E_NEWLIST=506

# --- Error IDs ------------------ #
const ID_VARMISSING=1
const ID_SECTIONMISSING=2
const ID_ENV_NOTROOT=3
const ID_VARNAMES=4
const ID_VARNOTEXP=5
const ID_TDATAPERM=6
const ID_TESTSPERM=7
const ID_DESCPERM=8
const ID_TESTPERM=9
const ID_FILENAMEMISSING=10

# --- Functions ------------------ #
function check_env () {
    if (( $(id -u) != $ROOT_UID )); then
	report 'ERROR' $ID_ENV_NOTROOT 'Tests are not executed under root'
	return ${E_FAIL}
    fi
    
    # selftest of selftest :]
    if [[ -z "${VAR_NAMES}" ]]; then
        report 'ERROR' $ID_VARNAMES "Variable \"VAR_NAMES\" is empty or not exported."
        return ${E_FAIL}
    fi
    
    for VAR_NAME in $VAR_NAMES; do
	local OUT="`eval 'echo $'${VAR_NAME}`"
	if [[ -n "${OUT}" ]]; then
	    continue
	else
	    report 'ERROR' $ID_VARNOTEXP "Variable \"${VAR_NAME}\" is empty or not exported."
	    return ${E_FAIL}
	fi
    done
    
    return ${E_OK}
}

function get_header () {
    sed -n '/\[HEADER\]/,/\[[a-zA-Z0-9_]*\]/ s/^\s*\([^\[]*=.*\)$/\1/p' "${1}"
}

# <env> <header>
function get_env () {
    echo "${2}" | sed -n 's/^[[:space:]]*'"${1}"'=["]\{0,1\}\([^"]*\)["]\{0,1\}[[:space:]]*$/\1/p'
}

function check_perms () {
    local E_RET=${E_OK}

    # dirs
    check_file_perm "${TDATA_DIR}" ${TDATA_PERM} ${TDATA_OWNER} 1 $ID_TDATAPERM "Main permanent data directory" || E_RET=${E_FATAL}
    check_file_perm "${TESTS_DIR}" ${TESTS_PERM} ${TESTS_OWNER} 1 $ID_TESTSPERM "Tests directory" || E_RET=${E_FATAL}
    
    # scripts + dscs
    for DESCFILE in `find ${TESTS_DIR} -name '*.dsc'`; do
	local HEAD="`get_header \"${DESCFILE}\"`"
	local TESTFILE="`get_env FILE_NAME \"${HEAD}\"`"
	local TESTNAME="`get_env NAME \"${HEAD}\"`"
	local NDATADIR="`get_env NEED_DATA_DIR \"${HEAD}\"`"

	if [[ -z "${NDATADIR}" ]]; then
		NDATADIR=0
	fi

	if [[ -z "${TESTFILE}" || -z "${TESTNAME}" ]]; then
		report 'ERROR' $ID_FILENAMEMISSING "FILE_NAME or NAME missing or empty in ${DESCFILE}"
		return ${E_FATAL}
	fi
	
	check_file_perm "${DESCFILE}" ${DESC_PERM} ${DESC_OWNER} 1 $ID_DESCPERM "Test description file" || E_RET=${E_FATAL}
	check_file_perm "${TESTS_DIR}/${TESTFILE}" ${TEST_PERM} ${TEST_OWNER} 1 $ID_TESTPERM "Test script" || E_RET=${E_FATAL}
	if (( $NDATADIR == 1 )); then
		check_file_perm "${TDATA_DIR}/${TESTNAME}" ${TDATA_PERM} ${TDATA_OWNER} 1 $ID_TDATAPERM "Permanent data directory" || test ${E_RET} != ${E_FATAL} && E_RET=${E_FAIL}
	fi

	check_dsc "${DESCFILE}"
	case "$?" in
		${E_FAIL})
			if (( ${E_RET} != ${E_FATAL} )); then
				E_RET=${E_FAIL}
			fi
		;;
		${E_FATAL})
			E_RET=${E_FATAL}
		;;
	esac
    done

    return ${E_RET}
}

function check_vars () {
    for VAR in ${3}; do
	if [[ -z "`get_env \"${VAR}\" \"${1}\"`" ]]; then
	    report 'ERROR' $ID_VARMISSING "${VAR} is missing or empty in ${2}"
	fi
    done
}

function get_sections () {
    L="`get_env LEVELS \"${1}\"`"

# DISABLED for now
#    echo "DEFAULT"
#    for lnum in ${L}; do
#	echo "LEVEL_${lnum}"
#    done
}

function get_level_env () {
    sed -n '/^[[:space:]]*\['"${2}"'\][[:space:]]*$/,/^[[:space:]]*\[[A-Za-z0-9_]*\][[:space:]]*$/ s/^[[:space:]]*\([^]]*=.*\)$/\1/p' "${1}"
}

# <file>
function check_dsc () {
    HEAD="`get_header \"${1}\"`"
    
    check_vars "${HEAD}" "${1}" "NAME FILE_NAME DESCRIPTION LEVELS GROUPS" || return ${E_FATAL}
    
    for SECTION in `get_sections "${HEAD}"`; do
	if [[ -z "`get_level_env \"${1}\" \"${SECTION}\"`" ]]; then
	    report 'WARNING' $ID_SECTIONMISSING "${SECTION} section is missing in ${1}"
	    return ${E_FAIL}
	fi
    done
    
    return ${E_OK}
}

# --- Main ----------------------- #

check_env   || (test $? == ${E_FATAL} && test_exit ${E_FATAL} "Selftest failed")
check_perms || (test $? == ${E_FATAL} && test_exit ${E_FATAL} "Selftest failed")

test_exit ${E_OK}
# --- EOF ------------------------ #
